#!/usr/bin/env python
# -*- coding: UTF-8 -*-

###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# ajout_utilisateur.py
#
# Utilitaire d'ajout d'utilisateur dans un annuaire LDAP
#
###########################################################################

#####################
# modules importé
#####################
import getopt
import sys
from horus import backend
from os import system

#####################
# variables globales
#####################

###########################
# définition des fonctions

# fonction de création d'un groupe
def creation_groupe(groupe):
    """
    création de groupe avec demande 
    """
    reponse = raw_input(" Le groupe %s n'existe pas, voulez-vous le créer (O/N) ? (N): " % groupe)
    if reponse == "":
        reponse = "N"
    if reponse[0].upper() == 'O':
        backend.add_group(groupe)
        return 1
    else:
        return 0

# fonction de lecture d'un utilisateur en ligne de commande ou dans un fichier
def read_user(user):
    """user : définition de l'utilisateur sous la forme :
        login;grp1,grp2;lecteur;passwd
    """
    data = user.split(';')
    if len(data) != 4:
        print "Définition de l'utilisateur invalide !"
        sys.exit("Forme attendue : \"login;grp1[,grp2];lecteur;passwd\"")
    lst_groupes = data[1].split(',')
    return data[0], lst_groupes, data[2], data[3],

# saisie interactive d'un utilisateur
def saisie_inter():
    print ("\n saisie d'un utilisateur en mode interactif \n")
    print ("Les valeurs entre parenthèses sont prises par défaut\n\n");

    # Saisie des informations
    # login, home(/home/login), smbhome(\\\\eole\\login), script(login.bat)
    # profile(\\\\eole\\profiles\\login), groupe, lettre de lecteur(U:)

    # login
    login = raw_input(" Veuillez saisir le login de l'utilisateur : ")

    # on passe le nom en minuscule (pb à la connexion)
    login = login.lower()

    if backend.is_system_user(login):
        sys.exit("Erreur, nom d'utilisateur reservé")

    if backend.is_user(login):
        sys.exit("Erreur, l'utilisateur existe déjà")

    # groupe
    groupe = raw_input(" Entrez le groupe principal de l'utilisateur : ")
    temp_grp = []

    # groupes supplémentaires, on crée une liste temporaire des groupes souhaités
    while (groupe != ""):
        temp_grp.append(groupe)
        groupe = raw_input (" Groupe supplémentaire (<entrée> pour terminer la liste) : ")
    
    #répertoire unix à créer (toujours par défaut)
    #home = ""
    #home = raw_input(" Chemin du répertoire utilisateur (/home/"+login+") : ")
    #if home == "":
    #    home = homebase+"/"+login

    #répertoire samba (idem)
    #smbhome = ""
    #smbhome = raw_input(" Chemin du répertoire samba (\\\\\\\\eole\\\\"+login+") : ")
    #if smbhome == "":
    #    smbhome = "\\\\\\\\"+smb_serveur+"\\\\"+login+"\\\\perso"

    #script de lancement (idem)
    #script = ""
    #script = raw_input(" script de connexion ("+login+".bat) : ")
    #if script == "":
    #    script = login+".bat"

    # profiles itinérants
    profiles = 0
    rep_valid = 0
    while (rep_valid == 0):
        use_profile = raw_input(" Voulez vous activer les profils itinérants ? (N) : ")
        if use_profile.upper() == 'O':
            rep_valid = 1
            #profiles = "\\\\\\\\"+smb_serveur+"\\\\"+login+"\\\\profiles"
            profiles = 1
        elif use_profile.upper() == 'N' or use_profile == "":
            rep_valid = 1

    rep = raw_input("utilisateur actif oui/non [oui] : ")
    if rep.lower in ["n", "non"]:
        actif = False
    else:
        actif = True

    # lettre de lecteur
    drive = ""
    drive = raw_input(" Lettre du lecteur pour le montage du répertoire (U:) : ")
    if drive == "":
        drive = "U:"

    ngrp = 0
    groupes = []
    groupe_princ = ""
    # Vérification de la liste des groupes et création de la liste finale
    for groupe in temp_grp:

        groupe_ok = False
        
        while (not groupe_ok):

            if not backend.is_group(groupe):
                result = creation_groupe(groupe)
                if (result == 0):
                # on ne veut pas créer le groupe
                # saisie d'un autre groupe
                    groupe = ""
                    groupe = raw_input (" Entrez un autre nom de groupe\n (rien pour ne pas le remplacer) : ")
                    if groupe == "":
                        groupe_ok = True
                        
            else:
                # le groupe a été trouvé dans la base
                groupe_ok = 1
                # mise en place des paramètres
                if ngrp == 0:
                    # groupe principal de l'utilisateur
                    groupe_princ = groupe
                else:
                    # groupes supplémentaires suivants
                    groupes.append(groupe)
                # on passe au groupe suivant
                ngrp += 1
                
    # si il ne reste aucun groupe, on ne crée pas l'utilisateur
    if groupe_princ == "":
        sys.exit("erreur, pas de groupe principal pour cet utilisateur !")

    # ajout de l'utilisateur
    result = backend.add_user(login, groupe_princ, groupes, drive, profiles)

    if not result:
        #l'ajout de l'utilisateur a echoué
        sys.exit("erreur, l'enregistrement de l'utilisateur a échoué")
    else:
        if groupes == []:
            print("\n\n utilisateur "+login+" créé et ajouté au groupe "+groupe_princ+"\n")
        else:
            print("\n\n utilisateur "+login+" créé et ajouté au groupes "+groupe_princ+" et "+str(groupes)+"\n")

    if not actif:
        backend.set_active_users(login, False)

    # mise à jour du mot de passe
    code_retour = 1
    while code_retour != 0:
        code_retour = system ("/usr/sbin/smbldap-passwd "+login)
        if code_retour != 0 and code_retour != 2560:
            sys.exit("\nerreur lors de la mise à jour du mot de passe (erreur ldap ?)\n")
    

# saisie par l'intermédiaire d'un fichier ou en ligne de commande
def saisie_non_inter(login,lst_grp,drive,passwd=None,actif=0,use_profile=0,newgroups=0):
    """ traitement d'un utilisateur dans un fichier ou en ligne de commande
    par défaut, les utilisateurs ne sont pas actifs, les profils itinérants sont
    désactivés, et les groupes non existants ne sont pas créés
    """

    rapport =""
        
    # on crée une liste des groupes
    temp_grp = lst_grp

    # profiles (idem)
    if use_profile == 0:
        profiles = 0
    else:
        profiles = 1

    # Vérification de la liste des groupes et création de la liste finale
    liste_grp = []
    ngrp = 0
    if len(temp_grp) == 0:
        return("erreur, aucun groupe défini pour l'utilisateur "+login+" !")
    else:
        for groupe in temp_grp:

            if not backend.is_group(groupe):
                # selon la variable 'newgroups', on va soit créer le groupe manquant,
                # soit afficher un message d'avertissement
                if newgroups == 0:
                    # warning
                    if __name__ == '__main__':
                        print "attention, groupe "+groupe+" inexistant (utilisateur "+login+")"
                    else:
                        rapport += "attention, groupe "+groupe+" inexistant (utilisateur "+login+")\n"
                        
                    # on retire le groupe de la liste
                    temp_grp.remove(groupe)
                else:
                    # création du groupe manquant
                    retour = backend.add_group(groupe)
                        
            # mise en place des paramètres
            if ngrp == 0:
                # groupe principal de l'utilisateur
                groupe_princ=groupe
            #elif ngrp == 1:
            #    # 1er groupe supplémentaire
            #    liste_grp = groupe
            else:
                # groupes supplémentaires suivants
                liste_grp.append(groupe)
            # on passe au groupe suivant
            ngrp += 1

    retour = backend.add_user(login, groupe_princ, liste_grp, drive, profiles)
                
    if __name__ != '__main__':
        if not retour:
            rapport += "erreur pour "+login+" (utilisateur existant ou groupe principal inconnu ?)\n"
        else:
            rapport += "utilisateur "+login+" créé\n"

    if actif == 0:
        backend.set_active_users(login, False)

    if passwd is None:
        # par défaut, mot de passe=login si non donné (définir une autre politique ?)
        passwd = login

    code_retour = backend.mod_password(login, passwd)
    if not code_retour:
        return("\nerreur lors de mise à jour du mot de passe (utilisateur "+login+")\n")

    if rapport == "":
        return "OK"
    else:
        return rapport


# ajout de plusieurs utilisateurs depuis un fichier
def traite_fichier(fic,actif,use_profile,newgroups):
    """traite les lignes du buffer contenant une liste d'utilisateurs"""
    
    rapport = ""
    # on place le contenu du fichier dans un tableau
    for user in fic:
        if user.startswith('#'):
            # on ne traite pas les lignes de commentaires
            pass
        else:
            (login,lst_groups,drive,passwd)=read_user(user)
            # on passe le nom en minuscule (pb à la connexion)
            login = login.lower()
            # vérification du nom d'utilisateur
            if backend.is_system_user(login):
                if __name__== '__main__':
                    sys.exit("Erreur, le nom d'utilisateur "+login+" est interdit (compte système)")
                else :
                    return("Erreur, le nom d'utilisateur "+login+" est interdit (compte système)")
            else:
                print "création de l'utilisateur "+login+" ..."
                retour = saisie_non_inter(login,lst_groups,drive,passwd,actif,use_profile,newgroups)
                if retour != "OK":
                    if __name__ == '__main__':
                        #execution en ligne de commande
                        sys.exit(retour)
                    else:
                        # appel depuis un autre programme python
                        rapport += retour
                    
    if __name__ != '__main__':
        if  rapport != "":
            return rapport
        else:
            return "OK"
        
    
####################
# boucle principale
def main():
    
    f_in = None
    actif = 0
    use_profile = 0
    newgroups = 0
    user = ""
    
    try:
        options, args = getopt.getopt(sys.argv[1:],'f:u:apgh',
                                 ['fichier=',
                                  'user=',
                                  'actif',
                                  'profile',
                                  'groupes_auto',
                                  'help'])
    except getopt.GetoptError:
        # on récupère les information de l'exception
        er_type, er_value, er_traceback = sys.exc_info()
        er_msg = str(er_value)
        # on prévient l'utilisateur que l'option nécessite un argument
        sys.exit("l'option -%s est inconnue ou nécessite un argument" % er_msg[er_msg.index('-')+1])

    for (opt,val) in options:
        # nom du fichier de données
        if (opt == '-f') or (opt == '--fichier'):
            f_in = val

        # définition de l'utilisateur
        if (opt == '-u') or (opt == '--user'):
            user = val

        # utilisateur activé ou non
        elif (opt == '-a') or (opt == '--actif'):
            actif = 1

        # profil itinérants activés ou non
        elif (opt == '-p') or (opt == '--profile'):
            use_profile = 1

        # création automatique des groupes manquants
        elif (opt == '-g') or (opt == '--groupes_auto'):
            newgroups = 1

        # appel de l'aide
        if (opt == '-h') or (opt == '--help'):
            sys.exit("""options possibles : -f fichier de données
                    -a utilisateur actif
                    -p profils itinérants actifs
                    -g créer automatiquement les groupes manquants
                    -u définition de l'utilisateur ("login;grp1,grp2,..,grpn;lettre;passwd")
                    -h aide (ce message)
                     """)

    if (f_in is not None) and (user != ""):
        sys.exit("""Erreur, la création par fichier et ligne de 
commande sont mutuellement exclusives""")

    if f_in is not None:
        # traitement d'un fichier de données
        try:
            f_in=open(f_in,"r")
        except:
            sys.exit("erreur, le fichier spécifié ne peut pas être lu")
        else:    
            # récupération du fichier en mémoire
            buffer_fic = f_in.readlines()
            f_in.close()
            traite_fichier(buffer_fic,actif,use_profile,newgroups)
    else:
        # on regarde si la ligne de commande comporte la définition de l'utilisateur
        if user == "":
            saisie_inter()
        else:
            # on récupère les données de l'utilisateur
            (login,lst_groups,drive,passwd)=read_user(user)
            # on passe le nom en minuscule (pb à la connexion)
            login = login.lower()
            # on vérifie que le nom d'utilisateur est valide
            if backend.is_system_user(login) :
                sys.exit("Erreur, ce nom d'utilisateur est interdit (compte système)")
            else:
                # on le crée
                saisie_non_inter(login,lst_groups,drive,passwd,actif,use_profile,newgroups)


# si on a appelé le programme directement : on éxecute main
if __name__ == "__main__":
    main()
