#!/usr/bin/python
# -*- coding: UTF-8 -*-

###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# manage_special_shares.py
#
# Utilitaire de gestion des groupes spéciaux
#
###########################################################################

###################
# modules importé
from horus import backend
import getopt, sys
#####################

# si appelé en ligne de commande...
def main():
    # définition du message d'aide
    help_msg ="""arguments:
         -p (--partage) nom du partage
         -l (--lettre) lettre sur laquelle connecter le partage sous windows
         -h (--help) (ce message)

     exemples : 
     
     * ajout/modification de la lettre de partage : ./manage_special_shares.py -p partage -l J:
     * suppression de la lettre réservée          : ./manage_special_shares.py -p partage -l ""
     * récupération de la lettre associée à un partage : ./manage_special_shares.py -p partage
     
     """

    # récupération des arguments de la ligne de commande
    try:
        options, args = getopt.getopt(sys.argv[1:],'p:s:l:m:h',
                                 ['partage=',
                                  'lettre=',
                                  'share=',
                                  'modele='
                                  'help'])
    except getopt.GetoptError:
        # on récupère les information de l'exception
        er_type, er_value, er_traceback = sys.exc_info()
        er_msg = str(er_value)
        # on prévient l'utilisateur que l'option nécessite un argument
        sys.exit("l'option -%s est inconnue ou nécessite un argument" % er_msg[er_msg.index('-')+1])

    partage = None
    letter = None
    share = None
    modele = None

    for (opt,val) in options:
        if (opt == '-p') or (opt == '--partage'):
            partage = val

        elif (opt == '-l') or (opt == '--lettre'):
            letter = val

        elif (opt == '-s') or (opt == '--share'):
            share = val

        elif (opt == '-m') or (opt == '--modele'):
            modele = val

        elif (opt == '-h') or (opt == '--help'):
            sys.exit(help_msg)

    # sans arguments => affichage des lettres
    if partage is None and share is None:
        dico = backend.get_special_shares()
        for key,value in dico.items():
            print '%s %s'%(value,key)
        sys.exit()

    if partage is not None:
        if letter is None:
            # seulement le nom => retourne la lettre
            dico = backend.get_special_shares()
            if dico.has_key(partage):
                print dico[partage]
            else:
                print ""
            sys.exit()
        else:
            # nom et lettre => enregistrement
            backend.mod_share(partage, drive=letter)
            sys.exit()
    
    if modele is None:
        if share == 'ALL':
            for model in backend.get_share_templates():
               print model
        else:
            try:
                res = backend.ldapsearch("(&%s(cn=*/%s))" % (backend.share_filters, share), ['sambaShareModel'])
                print res[0][1]['sambaShareModel'][0]
            except:
                print ''
        sys.exit()
    else:
        backend.mod_share(share, model=modele)
        sys.exit()


if __name__ == '__main__':
    main()
