#! /usr/bin/env python
# -*- coding: UTF-8 -*-

###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# dyn-logon.py
#
# Script de génération de fichiers netlogon depuis LDAP
#
###########################################################################

import commands
from optparse import OptionParser
from horus import backend
import os
import sys
__version__ = "0.99"


retour = chr(13)+chr(10)


def special_share(netuse, name, drive, uri):
    """
    définition d'un partage avec lettre de lecteur résevée
    """
    script = ""
    script+= "%sIF EXIST %s\\nul goto err%s" % (retour, drive, name)
    script+= "%s%s %s %s" % (retour, netuse, drive, uri)
    script+= "%sgoto suit%s" % (retour, name)
    script+= "%s:err%s" % (retour, name)
    script+= "%s%s * %s" % (retour, netuse, uri)
    script+= "%s:suit%s%s" % (retour, name, retour)
    return script

def get_share_attr(share):
    """renvoie "groupe", "name", "uri", "path", "drive"=None
    """
    groupe = share[1]['sambaShareGroup'][0]
    name = share[1]['sambaShareName'][0]
    uri = share[1]['sambaShareURI'][0]
    path = share[1]['sambaFilePath'][0]
    try: drive = share[1]['sambaShareDrive'][0]
    except: drive = None
    return groupe, name, uri, path, drive

def esu(netuse):
    """
    code a ajouter dans le script utilisateur pour l'execution d'Esu
    """
    from creole import parsedico
    d = parsedico.parse_dico()
    script = ""
    script += "%sIF EXIST R:\\nul goto erresu" % (retour)
    script += "%s%s R: \\\\%s\\icones$" % (retour, netuse, d['smb_netbios_name'])
    script += "%s:erresu" % (retour)
    script += "%s\\\\%s\\ESU\\Base\\esuclnt.exe%s" % (retour, d['smb_netbios_name'], retour)
    return script

def gen_groupedir(user, user_group_dir):
    """Suppression du répertoire contenant les liens vers les groupes de l'utilisateur
    et (re)construction du répertoire
    """
    # visiblement os.removedirs()  n'est pas si récursif que ça !
    os.system('rm -rf %s'%user_group_dir)
    os.makedirs(user_group_dir)
    os.system('/bin/chown %s %s'%(user, user_group_dir))
    os.system('setfacl -bk %s'%user_group_dir)
    os.chmod(user_group_dir,0500) # pas d'écriture à la racine  S:

def get_scripts(user,groups,machine,os_type):
    """
    gestion des scripts bat personnalisés
    +--path_scripts/users/<user>.bat
    +--path_scripts/groups/<group>.bat
    +--path_scripts/machines/<machine>.bat
    +--path_scripts/os/<os>.bat
    +--path_scripts/os/<os>/<group>.bat
    +--path_scripts/os/<os>/<user>.bat
    """
    motif_include = "%%NetUse%%"
    path_scripts = '/home/netlogon/scripts'
    buffer_debut = ""
    buffer_fin = retour
    # on créé la liste des fichiers possibles
    chemins = []
    chemins.append(os.path.join(path_scripts, 'users', '%s.bat'%user))
    chemins.append(os.path.join(path_scripts, 'machines', '%s.bat'%machine))
    for group in groups:
        chemins.append(os.path.join(path_scripts, 'groups', '%s.bat'%group))
        chemins.append(os.path.join(path_scripts, 'os', os_type, '%s.bat'%group))
    chemins.append(os.path.join(path_scripts, 'os', '%s.bat'%os_type))
    chemins.append(os.path.join(path_scripts, 'os', os_type, '%s.bat'%user))
    # on traite ceux qui sont présents
    for chemin in chemins:
        if os.path.isfile(chemin):
            fic = open(chemin,"r")
            debut = 1
            for line in fic.readlines():
                if line.strip() == motif_include:
                    debut = 0
                else:
                    if debut == 1:
                        # ajout de la ligne au fichier (+CR/LF pour windows)
                        buffer_debut += line.strip()+retour+retour
                    else:
                        buffer_fin += line.strip()+retour+retour
    return (buffer_debut,buffer_fin)

def parse_command_line():
    """
    traitement de la ligne de commande
    option.user
    option.machine
    option.os
    option.ip
    """
    parser = OptionParser(__doc__, version='%%prog version %s' % __version__)

    parser.add_option('-u', '--user', action='store', dest='user',
                      help='login', metavar='LOGIN')
    parser.add_option('-o', '--os', action='store', dest='os',
                      help="type d'OS", metavar='TYPE_OS',
                      choices=['Win95', 'Win2K', 'WinXP', 'Win2K3'])
    parser.add_option('-m', '--machine', action='store', dest='machine',
                      help="nom de machine", metavar='NOM_MACHINE')
    parser.add_option('-i', '--ip', action='store', dest='ip',
                      help="adresse ip", metavar='ADR_IP')

    options, args = parser.parse_args()

    if len(args) >= 1:
        parser.error("le script ne prend pas d'argument")
    if options.user is None:
        parser.error("pas de nom de login")
    if options.machine is None:
        parser.error("pas de nom de machine")
    if options.os is None:
        parser.error("pas de type d'OS")
    if options.ip is None:
        parser.error("pas de type d'IP")

    return options

def main():
    script = ""
    tmpscript = ""
    # gestion de la ligne de commande
    opt = parse_command_line()

    # données de l'utilisateur
    try:
        user_data = backend.ldapsearch("(&%s(uid=%s))" % (backend.user_filters, opt.user))[0][1]
    except:
        sys.exit("Utilisateur %s non trouvé" % opt.user)

    if opt.os == 'Win95':
        netuse = 'NET USE'
    else:
        netuse = 'NET USE /PERSISTENT:NO' 

    user_groups = backend.get_user_groups(opt.user)

    # le répertoire "$HOME/groupes"
    user_group_dir = os.path.join(user_data['homeDirectory'][0], 'groupes')
    # création du répertoire
    gen_groupedir(opt.user, user_group_dir)

    shares = backend.ldapsearch(backend.share_filters)
    
    for share in shares:
        groupe, name, uri, path, drive = get_share_attr(share)
        if name == 'icones$': continue
        if groupe in user_groups:
            if not drive:
                os.symlink(path, os.path.join(user_group_dir, name))
                # pas de lettre -> montage en dernier
                # tmpscript += "%s%s * %s" % (retour, netuse, uri)
            else:
                script += special_share(netuse, name, drive, uri)

    if opt.os == 'Win95':
        # répertoire personnel explicite
        script += special_share(netuse, 'perso', user_data['sambaHomeDrive'][0], '\\\\%s\\PERSO'%backend.smb_serveur)
    
    # ajout des partages "simples"
    script += tmpscript

    if backend.use_esu():
        # instructions pour Esu
        script += esu(netuse)
    
    # scripts perso
    debut,fin = get_scripts(opt.user, user_groups, opt.machine, opt.os)

    batfile = "/home/netlogon/%s" % user_data['sambaLogonScript'][0]
    try:
        fic = open(batfile,"w")
        fic.write('%s%s%s'%(debut,script,fin))
        fic.close()
        os.system("chown %s %s" % (opt.user, batfile))
        os.system("chmod 755 %s" % batfile)
    except:
        sys.exit("erreur d'écriture du fichier %s" % batfile)

    sdate = commands.getoutput('date +%a" "%d" "%b" "%Y" "%H":"%M').capitalize()
    cmd = 'echo CONNECTION %s %s %s %s %s'%(sdate, opt.user, opt.machine, opt.os, opt.ip)
    os.system('%s >> /var/log/samba/connexions.log'%(cmd))
    os.system('%s >> /var/log/samba/log.%s'%(cmd, opt.machine))
    

if __name__ == '__main__':
    try: main()
    except Exception, e: file('/tmp/dyn-logon.errors','w').write('%s'%e)

