#! /usr/bin/env python
# -*- coding: utf-8 -*-

###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# serveur.py
#
# Service pour la gestion de l'annuaire LDAP du serveur Horus
#
###########################################################################

import sys
from horus import backend

from os import environ
from os.path import join

from OpenSSL import SSL
from twisted.internet import defer,protocol,reactor
from twisted.web import xmlrpc,server
import xmlrpclib, StringIO

# librairies et variables nécessaires au cryptage du mot de passe
import string
import md5

DES_SALT = list('./0123456789' 'ABCDEFGHIJKLMNOPQRSTUVWXYZ' 'abcdefghijklmnopqrstuvwxyz')

#################################################
## Factory permettant de créer un contexte SSL ##
#################################################
class ServerContextFactory:

    def getContext(self):
        """Create an SSL context.
        This is a sample implementation that loads a certificate from a file 
        called 'server.pem'."""
        ssl_dir = '/etc/ssl/horus'
        ctx = SSL.Context(SSL.SSLv23_METHOD)
        try:
            ctx.use_certificate_file(join(ssl_dir, 'server.crt'))
            ctx.use_privatekey_file(join(ssl_dir, 'server.key'))
        except:
            sys.exit("erreur, les certificats n'ont pas été trouvés dans %s"%ssl_dir)
        return ctx
    
#########################################
## Fonctions de cryptage du mot de passe

def _to64(v, n):
    r = ''
    while (n-1 >= 0):
        r = r + DES_SALT[v & 0x3F]
        v = v >> 6
        n = n - 1
    return r

def shadow_crypt(passwd, salt=None, magic='$1$'):
    """Encrypt passwd with MD5 algorithm."""
    import time
    if not salt:
        salt = repr(int(time.time()))[-8:]
    elif salt[:len(magic)] == magic:
        # remove magic from salt if present
        salt = salt[len(magic):]

    # salt only goes up to first '$'
    salt = string.split(salt, '$')[0]
    # limit length of salt to 8
    salt = salt[:8]

    ctx = md5.new(passwd)
    ctx.update(magic)
    ctx.update(salt)
    
    ctx1 = md5.new(passwd)
    ctx1.update(salt)
    ctx1.update(passwd)
    
    final = ctx1.digest()
    
    for i in range(len(passwd), 0 , -16):
        if i > 16:
            ctx.update(final)
        else:
            ctx.update(final[:i])
    
    i = len(passwd)
    while i:
        if i & 1:
            ctx.update('\0')
        else:
            ctx.update(passwd[:1])
        i = i >> 1
    final = ctx.digest()
    
    for i in range(1000):
        ctx1 = md5.new()
        if i & 1:
            ctx1.update(passwd)
        else:
            ctx1.update(final)
        if i % 3: ctx1.update(salt)
        if i % 7: ctx1.update(passwd)
        if i & 1:
            ctx1.update(final)
        else:
            ctx1.update(passwd)
        final = ctx1.digest()
    
    rv = magic + salt + '$'
    final = map(ord, final)
    l = (final[0] << 16) + (final[6] << 8) + final[12]
    rv = rv + _to64(l, 4)
    l = (final[1] << 16) + (final[7] << 8) + final[13]
    rv = rv + _to64(l, 4)
    l = (final[2] << 16) + (final[8] << 8) + final[14]
    rv = rv + _to64(l, 4)
    l = (final[3] << 16) + (final[9] << 8) + final[15]
    rv = rv + _to64(l, 4)
    l = (final[4] << 16) + (final[10] << 8) + final[5]
    rv = rv + _to64(l, 4)
    l = final[11]
    rv = rv + _to64(l, 2)
    
    return rv

# fonction de conversion des objets unicodes en string (récursif)
def convert(objet):
    """Transforme les objets unicode contenus dans un objet en chaines
    """
    if type(objet) == tuple:
        l = []
        for item in objet:
            l.append(convert(item))
        return l
    if type(objet) == list:
        l = []
        for item in objet:
            l.append(convert(item))
        return l
    if type(objet) == dict:
        dico={}
        for cle in objet.keys():
            dico[cle] = convert(objet[cle])
        return dico
    if type(objet) == unicode:
        string =  objet.encode('ISO-8859-1')
        return string
    return objet


def convert_None(var):
    """
    Si la variable est initialisée avec la chaîne None, on renvoit None
    """
    if var == "None":
        return None
    else:
        return var


# fonctions spéciales ajoutée pour la gestion 
# des profils non administrateur
def user_in(user, groups, myself=''):
    """
    test l'appartenance de l'utilisateur à un groupe
    """
    if user == 'admin':
        return False    
    usergroups = backend.get_user_groups(user)
    for g in groups:
        if g in usergroups:
            if user == myself:
                # soi-même
                return True
            if backend.get_attr(user, 'employeeType') != []:
                # autres admin délégués
                return False
            return True
    return False

def g_filtre(groupe):
    """
    suppression des groupes spéciaux
    d'une liste de groupes
    """
    for g_special in ['DomainAdmins','DomainUsers','DomainComputers']:
        if g_special in groupe: groupe.remove(g_special)
    return groupe

class autorisationError(Exception):
    pass
    

##############################################
## Surclasse du serveur XMLRPC twisted pour ##
## permettre l'authentification             ##
##############################################
class XMLRPCSecure(xmlrpc.XMLRPC):
    
    
    def __init__(self):
        xmlrpc.XMLRPC.__init__(self)
        # FIXME définition du mot de passe pour test
        #self.user = 'administrateur'
        # on récupère le mot de passe dans /etc/shadow
        #print "Définition des mots de passe ..."
        # définition des autorisations pour chaque utilisateur
        self.authorizations = {}
        # administrateur de l'appli
        self.authorizations['administrateur'] = ['get_users','get_groups','get_machines','get_printersIp','get_usergroups',
                                  'inscrit', 'get_share_templates', 'get_share_model',
                                  'import_users','add_user','del_user','get_user_data','mod_user','desinscrit','get_special_shares',
                                  'get_shares','get_groupusers','add_share','del_share','add_group','del_groupe','synchro',
                                  'mod_share', 'add_machine','del_machine','del_printer','set_quota_limits','get_quota_limits',
                                  'get_my_profil', 'get_admin_groups', 'set_admin_groups']
        # administrateurs de groupes
        self.authorizations['admin_group'] = ['get_users', 'get_usergroups', 'get_user_data', 'get_groups', 
                                                'inscrit', 'desinscrit', 'mod_user',
                                                'set_quota_limits','get_quota_limits',
                                                'get_my_profil'] 

        # on récupère le mot de passe crypté pour horus
        #shadow = open ("/etc/shadow","r")
        #for ligne in shadow.readlines():
        #   if ligne.count(self.user)==1:
        #       self.password=ligne.split(':')[1]
               
        self.client_user = None
        self.client_pass = None
               

    def render(self,request):
        # test systématique de l'authentification
        cred_user = request.getUser()        
        cred_password = request.getPassword()
        # si le mot de passe ou le user envoyé n'est plus le même
        if (self.client_pass != cred_password) or (cred_password is None) or (self.client_user != cred_user) or (cred_user is None):
            # on le crypte et on le compare au mot de passe récupéré dans /etc/shadow
            #if (cred_user,shadow_crypt(cred_password,self.password)) != (self.user,self.password):
                # erreur d'authentification
            if not backend.authenticate(cred_user, cred_password):
                print "\nAuthentification eronée : %s sur %s"%(cred_user, request.host.host)
                return request.setResponseCode(401)
            else:
                print "\nclient authentifié : %s sur %s"%(cred_user, request.host.host)
                self.client_pass = cred_password
                self.client_user = cred_user
                self.profil = ''
                if cred_user == 'admin':
                    self.profil = 'administrateur'
                else:
                    self.groups = backend.get_attr(cred_user, 'employeeType')
                    if self.groups == []:
                        print "Aucun droit pour l'utilisateur %s"%(cred_user)
                        return request.setResponseCode(401)
                    else:
                        self.profil = 'admin_group'
            
        request.content.seek(0, 0)
        args, functionPath = xmlrpclib.loads(request.content.read())
        # conversion de l'unicode
        args_list=[]
        for arg in args:
            arg_conv=convert(arg)
            args_list.append(arg_conv)
        args = tuple(args_list)
        #try:
        if True:
            # on vérifie si l'utilisateur a le droit d'utiliser cette fonction
            if functionPath not in self.authorizations[self.profil]:
                print functionPath
                print "Utilisation de la fonction %s refusée pour %s (%s) : " % (str(functionPath), cred_user, request.host.host)
                return request.setResponseCode(401)
        #except KeyError:
        #    print "Aucun droit pour l'utilisateur %s"%(cred_user)
        #    return request.setResponseCode(401)        
        try:
            function = self._getFunction(functionPath)
        except xmlrpc.NoSuchFunction:
            self._cbRender(
                xmlrpclib.Fault(self.NOT_FOUND, "no such function %s" % functionPath),
                request
            )
        else:
            request.setHeader("content-type", "text/xml")
            defer.maybeDeferred(function, *args).addErrback(
                self._ebRender
            ).addCallback(
                self._cbRender, request
            )
        return server.NOT_DONE_YET

#############################################################
## Définition d'un protocole de gestion des processus fils ##
#############################################################

class HorusProcessProtocol(protocol.ProcessProtocol):

    def __init__(self, deferred):
       self.deferred = deferred
       self.s = StringIO.StringIO()
    def errReceived(self, text):
        self.s.write(text)
    def outReceived(self, text):
        self.s.write(text)
    def processEnded(self, reason):
        if self.deferred is not None:
            result = self.s.getvalue()
            self.deferred.callback(result)
                                                                                                                               
    
## Définition de la classe serveur HORUS ##
###########################################
#
# Cette classe est appellée à distance depuis le client graphique
# de gestion de l'annuaire. Toutes les fonctions liées à LDAP et
# a la configuration de samba sur le serveur sont exécutée sur le
# serveur même.
class horus_serv(XMLRPCSecure):
        
    
    ###########################
    ## Attribut de la classe ##
    ###########################
    
    # mot de passe LDAP
    # récupéré dans le fichier de configuration
    #__passwd = ldap_passwd

    ########################
    ## fonctions diverses ##
    ########################

    # profil du connecté
    def xmlrpc_get_my_profil(self):
        return self.profil

    def xmlrpc_get_admin_groups(self, user):
        return backend.get_attr(user, 'employeeType')

    def xmlrpc_set_admin_groups(self, user, groups):
        return backend.set_attr(user, 'employeeType', groups)    
 
    # synchronisation du serveur
    def xmlrpc_synchro(self):
        ## FIXME ##
        # exécution du script de synchronisation samba
        backend.synchronize()
        return "Synchronisation finie"

    # fonction bidon utilisée pour tester la connexion à l'objet
    def xmlrpc_test(self):
        pass

    # fonction de sauvegarde des partages spéciaux
    def xmlrpc_save_special_shares(self,dico_shares):
        backend.save_special_shares(dico_shares)
        
    def retour_ok(self,data):
        print data
        return "OK"

    def retour_nok(self,data):
        print data
        return "erreur lors de l'application des quotas"
    
    ## mise en place des quotas pour un utilisateur
    def xmlrpc_set_quota_limits(self,user,limit):
        if self.profil != 'administrateur':
            # l'utilisateur doit être dans mes groupes
            if not user_in(user, self.groups, self.client_user):
                raise autorisationError
        # calcul du nombre de blocs
        cmd = "/usr/share/ead/set_quota.py"
        d = defer.Deferred()
        p = HorusProcessProtocol(d)
        reactor.spawnProcess(p, cmd, [cmd, user, limit], env=environ, path=".")
        resultat=d.addCallbacks(self.retour_ok, self.retour_nok)
        return resultat


    ###############################
    ## Fonctions de consultation ##
    ###############################

    ## liste des utilisateurs
    def xmlrpc_get_users(self,serveur=None):
        try:
            if self.profil == 'administrateur':
                return backend.get_users()
            else:
                users = backend.get_users()
                allowed_users = []
                # on n'affiche que les membres des groupes autorisés
                for user in users:
                    if user_in(user, self.groups, self.client_user):
                        allowed_users.append(user)
                return allowed_users 
        except:
            return []

    ## liste des groupes
    def xmlrpc_get_groups(self,serveur=None):
        try:
            if self.profil == 'administrateur':
                return backend.get_groups()
            else:
                return g_filtre(backend.get_groups())
        except:
            return []

    def xmlrpc_get_share_templates(self):
        return backend.get_share_templates()

    ## liste des groupes associés (ou non) à un utilisateur
    def xmlrpc_get_usergroups(self,user,type,serveur):
        """
        liste des groupes d'un utilisateur
        si type=dispo : renvoi les groupes dans lesquels il n'est pas
        """
        if self.profil != 'administrateur':
            # l'utilisateur doit être dans mes groupes
            if not user_in(user, self.groups, self.client_user):
                raise autorisationError
            if type == "dispo":
                return g_filtre(backend.get_user_no_groups(user))
            else:
                return g_filtre(backend.get_user_groups(user))
        if type == "dispo":
            return(backend.get_user_no_groups(user))
        else:
            return(backend.get_user_groups(user))
                    
    ## liste des utilisateurs d'un groupe
    def xmlrpc_get_groupusers(self,group,serveur):
        return backend.get_members(group)
                

    ## récupération des données d'un utilisateur
    def xmlrpc_get_user_data(self,login,serveur):
        """récupére les données de l'utilisateur dans la base LDAP
           retourne la letter de lecteur, enable/disable, le groupe principal
        """
        if self.profil != 'administrateur':
            # l'utilisateur doit être dans mes groupes
            if not user_in(login, self.groups, self.client_user):
                raise autorisationError
        result_user = backend.ldapsearch("(&(objectClass=posixAccount)(uid="+login+"))")
        #recherche du groupe correspondant au gidNumber trouvé            
        result_groupe = backend.ldapsearch("(&(objectClass=posixGroup)(gidNumber="+result_user[0][1]['gidNumber'][0]+"))")
        groupe_principal = result_groupe[0][1]['cn'][0]
        try:
            if result_user[0][1]['sambaProfilePath'][0] != "":
                profile = 1
            else:
                profile = 0
        except KeyError:
            # la clé profilepath n'existe pas
            profile = 0
        #except:
        #    raise ValueError
        #else:
        #    return (result_user[0][1]['homeDrive'][0],result_user[0][1]['acctFlags'][0],groupe_principal,profile)
        return (result_user[0][1]['sambaHomeDrive'][0],result_user[0][1]['sambaAcctFlags'][0],groupe_principal,profile)

    ## récupération des limites de quota d'un utilisateur
    def xmlrpc_get_quota_limits(self,user):
        if self.profil != 'administrateur':
            # l'utilisateur doit être dans mes groupes
            if not user_in(user, self.groups, self.client_user):
                raise autorisationError
        cmd = "quota"
        d = defer.Deferred()
        p = HorusProcessProtocol(d)
        reactor.spawnProcess(p, cmd, [cmd, user], env=environ, path=".")
        resultat=d.addCallback(self.calc_quota)
        return resultat

    def calc_quota(self,output):
        lines=output.split('\n')
        try:
            nb_blocks = lines[2].split()[2]
        except IndexError:
            # pas de quotas pour cet utilisateur
            return 0
        else:
            # on retourne la limite en Mo
            limit = int(nb_blocks)/1024
            # si il y a des quotas mais inférieurs à 1 Mo -> 1 Mo
            if nb_blocks > 0 and nb_blocks < 1024:
                return 1
            else:
                return int(nb_blocks)/1024
    
    
    ## liste des ordinateurs autorisés sur le domaine
    def xmlrpc_get_machines(self,serveur):
        return backend.get_workstations()

    ## liste des imprimantes partagées
    def xmlrpc_get_printers(self,serveur):
        # on récupère la liste des utilisateurs
        ## FIXME
        return []
        #try:
        #    query=ldap.open(serveur)
        #    query.simple_bind_s("cn=admin, o=gouv, c=fr",self.__passwd)
        #    result_lp=query.search_s("o=gouv, c=fr", ldap.SCOPE_SUBTREE, \
        #                "(objectClass=sambaPhysicalPrinter)")
        #    query.unbind()
        #except:
        #    result_lp=["aucune imprimante trouvée"]

        ## création de la liste des noms récupérés
        #liste_lp = []
        #for indice in range(len(result_lp)):
        #    liste_lp.append(str(result_lp[indice][1]['cn'][0]))
        #
        #liste_lp.sort()
        #return liste_lp
    
    ## liste des imprimantes partagées avec leur adresse ip
    def xmlrpc_get_printersIp(self,serveur):
        # on récupère la liste des utilisateurs
        ## FIXME
        return []
        #try:
        #    query=ldap.open(serveur)
        #    query.simple_bind_s("cn=admin, o=gouv, c=fr",self.__passwd)
        #    result_lp=query.search_s("o=gouv, c=fr", ldap.SCOPE_SUBTREE, \
        #                "(objectClass=sambaPhysicalPrinter)")
        #    query.unbind()
        #except:
        #    result_lp=["aucune imprimante trouvée"]

        ## création de la liste des noms récupérés
        #liste_lp = []
        #for indice in range(len(result_lp)):
        #    liste_lp.append([str(result_lp[indice][1]['cn'][0]),str(result_lp[indice][1]['ipHostNumber'][0])])
        #
        #liste_lp.sort()
        #return liste_lp
    

    ## liste des partages dont on veut forcer la lettre
    def xmlrpc_get_special_shares(self,serveur):
        # on renvoie le dictionnaire des partages spécifiques
        return backend.get_special_shares()

    ###############################
    ## Fonctions de gestion ldap ##
    ###############################

    ## ajout d'un groupe
    def xmlrpc_add_group(self,nom_group):
        result = backend.add_group(nom_group)
        if not result:
            return "Erreur d'ajout (groupe existant ?)"
        else:
            return "OK"

    def xmlrpc_import_users(self,users,actifs,itinerants,groupes_auto):

        retour = ajout_utilisateur.traite_fichier(users,actifs,itinerants,groupes_auto)
        #os.unlink(chemin_temp)
        return(unicode(retour,'ISO-8859-1'))

        
    ## ajout d'un utilisateur
    def xmlrpc_add_user(self,login,groupe,drive,passwd,profile=0):

        # on passe le nom en minuscule (pb à la connexion)
        login = login.lower()

        if backend.is_system_user(login):
            return u"Erreur, nom d'utilisateur reservé"

        if backend.is_user(login):
            return u"Erreur, l'utilisateur existe déjà"

        result = backend.add_user(login, groupe, [], drive, profile)
        result = backend.mod_password(login, passwd)

        return "OK"
    
    
    ## modification d'un utilisateur
    def xmlrpc_mod_user(self,login,passwd="None",homedrive="None",enable="None",profile="None",gid="None"):
        """
        Modification d'un utilisateur existant.
        """
        if self.profil != 'administrateur':
            # l'utilisateur doit être dans mes groupes
            if not user_in(login, self.groups, self.client_user):
                raise autorisationError
        homedrive = convert_None(homedrive)
        enable = convert_None(enable)
        profile = convert_None(profile)
        gid = convert_None(gid)
        result = backend.mod_user(login, homedrive, enable, profile, gid)
        if not result:
            return ("erreur lors de la modification de l'utilisateur")
            
        if passwd != 'None':
            result = backend.mod_password(login, passwd)
            if not result:
                return "La modification du mot de passe dans l'annuaire a échoué"
        return "OK"
        

    ## supression d'un utilisateur
    def xmlrpc_del_user(self,login,mode):
        if mode != "systeme":
            result = backend.del_user(login, False)
        else:
            # suppression de l'utilisateur et de son répertoire
            result = backend.del_user(login, True)
        
        if not result:
            return "Erreur (utilisateur inexistant ?)"
        else:
            return "OK"
        
    ## inscription d'un utilisateur
    def xmlrpc_inscrit(self,user,groupes):
        if self.profil != 'administrateur':
            # l'utilisateur doit être dans mes groupes
            if not user_in(user, self.groups, self.client_user):
                raise autorisationError
        erreurs = ""
        for group in groupes:            
            retour = backend.inscription(user, group)
            if not retour:
                if erreurs != "":
                    erreurs += ","
                erreurs += group
                
        if erreurs != "":
            return "Erreur d'ajout : "+erreurs
        else:
            return "OK"

    ## desinscription d'un utilisateur
    def xmlrpc_desinscrit(self,user,groupes,serveur):
        if self.profil != 'administrateur':
            # l'utilisateur doit être dans mes groupes
            if not user_in(user, self.groups, self.client_user):
                raise autorisationError
        # vérification du groupe principal (on ne peut pas le supprimer)
        if type(groupes) != list:
            groupes=[groupes]
        gid = backend.get_gid(user)
        if gid in groupes:
            return "Impossible de supprimer le groupe principal %s" % gid
        erreurs = []
        for groupe in groupes:
            # supression effective du groupe             
            retour = backend.desinscription(user, groupe)
            if not retour:
                erreurs.append(groupe)
        if erreurs != []:
            return "Erreur lors de la désinscription à %s" % ",".join(erreurs)
        else:
            return "OK"

    def xmlrpc_get_shares(self,groupe,serveur):
        """Fonction de lecture des partages reliés à un groupe
        """
        try:
            result_shares = backend.ldapsearch("(&%s(sambaShareGroup=%s))"%(backend.share_filters, groupe))
        except:
            return "Erreur de lecture LDAP"
        else:
            liste_shares=[]
            for share in result_shares:
                liste_shares.append([share[1]['sambaShareName'][0],share[1]['sambaFilePath'][0]])
            return liste_shares

    def xmlrpc_get_share_model(self,sharename):
        """
        Renvoit le nom de modèle associé au partage
        """
        try:
            share = backend.ldapsearch("(&%s(description=%s))"%(backend.share_filters, sharename))
        except:
            return "Erreur de lecture LDAP"
        else:
            return share[0][1]['sambaShareModel'][0]

    ## ajout d'un partage samba pour un groupe
    def xmlrpc_add_share(self,sharename,groupe,serveur,filepath,drive="None",sticky=True,model='standard'):
        if drive == "None":
            drive = None
        # pas de synchronisation avec frontend
        ret = backend.add_share(sharename, groupe, filepath, drive, sticky, model, sync=False)
        if ret :
            return "OK"
        else:
            return "Erreur d'ajout du partage"
        
    # supression d'un partage dans l'annuaire
    def xmlrpc_mod_share(self,share_name, drive, model):
        """
        fonction de supression d'un partage dans l'annuaire LDAP
        """
        #try:
        print drive, model
        backend.mod_share(share_name, drive, model)
        #except:
        #    return "Erreur de modification dans LDAP"
        #else:
        return "OK"

    # supression d'un partage dans l'annuaire
    def xmlrpc_del_share(self,share_name,serveur):
        """fonction de supression d'un partage dans l'annuaire LDAP
        """
        try:
            backend.del_share(share_name, sync=False)
        except:
            return "Erreur de supression dans LDAP"
        else:
            return "OK"

    # supression d'un groupe
    def xmlrpc_del_groupe(self,groupe,serveur):
        """fonction de supression d'un groupe de l'annuaire
        """
        if not backend.is_group(groupe):
            return "erreur, le groupe n'existe pas"
        
        members = backend.get_members(groupe)
        if members != []:
            return "Erreur, ce groupe contient encore des utilisateurs"
        
        shares = backend.get_group_shares(groupe)
        if shares != []:
            return "Erreur, ce groupe contient encore des partages"

        if backend.del_group(groupe):
            return "OK"
        else:
            return "erreur lors de la supression"

    ## ajout d'une machine sur le domaine
    def xmlrpc_add_machine(self,nom_mach):
        result = backend.add_workstation(nom_mach)
        if not result:
            return "Erreur d'ajout (machine existante ?)"
        else:
            return "OK"

    ## suppression d'une machine sur le domaine
    def xmlrpc_del_machine(self,nom_mach):
        result = backend.del_workstation(nom_mach)
        if not result:
            return "Erreur de suppression (machine inexistante ?)"
        else:
            return "OK"

    ## ajout d'une imprimante
    def xmlrpc_add_printer(self,printer,addr_ip):
        result = 1
        if result != 0:
            return "Fonction non implémentée"
            #return "Erreur d'ajout (imprimante existante ?)"
        else:
            return "OK"
        
    ## suppression d'une imprimante
    def xmlrpc_del_printer(self,printer):
        result = 1 
        if result != 0:
            return "Fonction non implémentée"
            #return "Erreur de suppression (imprimante inexistante ?)"
        else:
            return "OK"



def main():

    # on récupère le nom du serveur
    from creole import parsedico
    Dico=parsedico.parse_dico()
    try:
        nom_serveur=Dico['nom_machine']
    except:
        sys.exit("Erreur, le nom de ce serveur n'est pas défini dans la configuration éole")
    
    print "mise en place du serveur xmlrpc sur le port 7080..."
    
    r=horus_serv()
    #xmlrpc.addIntrospection(r)
    reactor.listenSSL(7080, server.Site(r), ServerContextFactory())
    #reactor.listenTCP(7080, server.Site(r))
    reactor.run()

if __name__ == '__main__':
    main()
