# -*- coding: UTF-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# Parseur DOM pour fichiers XML issus de AAF
#
###########################################################################
import xml.dom.minidom
from horus import backend
import os

def get_text_node(node):
    """
    @param node: node minidom contenant du texte
    Utilitaire minidom permettant de récupérer le texte d'un node texte
    """
    texte = ""
    nodelist = node.childNodes
    for textnode in nodelist:
        if textnode.nodeType == textnode.TEXT_NODE:
            texte = textnode.data
    return texte

def gen_date(date):
    """jj/mm/aaaa -> aaaammjj"""
    date = date.replace('/','')
    date = date.replace('-','')
    date = date.replace(' ','')
    if len(date) != 8:
        return '00000000'
    else:
        return date

def manage_nom(nom):
    """ gestion des noms à particule """
    nom = nom.lower().replace("'", '')
    particules = ['de', 'du', 'dela', 'dos', 'di', 'es', 'el', 'le', 'la', 'da', 'van', 'ben', 'saint']
    for particule in particules:
        if nom.startswith('%s ' % particule):
            nom = nom.replace('%s ' % particule, particule, 1)
    nom = nom.split(' ')[0]
    nom = nom.split('-')[0]
    return nom

def gen_login(prenom, nom):
    prenom = prenom.lower().replace('-', '').split(' ')[0]
    nom = manage_nom(nom)
    if prenom != '':
        login = "%s.%s"%(prenom, nom)
    else:
        login = nom
    if len(login) > 20:
        login = login[0:20]
    return login.replace(' ','')

def ok_groupe(groupe):
    groupe = groupe.split()[0].lower()
    groupe = groupe.split('(')[0]     #parentheses
    groupe = groupe.replace(' ', '')  #espaces
    groupe = groupe.replace('*', 'e') #etoile, ex : mp*
    groupe = groupe.replace('.', '')  #point
    groupe = groupe.replace(':', '')  #2points
    groupe = groupe.replace('=', '')  #égal
    groupe = groupe.replace('"', '')  #double-quotes
    groupe = groupe.replace("'", '')  #quote
    # groupes 100% numériques
    if groupe.isdigit():
        # ajout du prefixe "type"
        groupe = "g"+groupe
    if len(groupe) > 19:
        groupe = groupe[0:20]
    return groupe

def parse_xml_file(filename, pwdtype="1", passfile="/tmp/passwords.csv"):
    """
    parsing du fichier XML AAF
    filename : fichier xml source
    pwdtype  : "1" pour date de naissance, "2" pour aléatoire
    passfile  : fichier résultat
    """
    passlines = []
    messages = []
    try:
        document = xml.dom.minidom.parse(filename)
    except:
        return ["Fichier invalide"]    
    root_node = document.documentElement
    for request in root_node.getElementsByTagName('addRequest'):
        for operational in request.getElementsByTagName('operationalAttributes'):
            attr = operational.getElementsByTagName('attr')[0]
            #print attr.getAttribute("name"), ":", get_text_node(attr.getElementsByTagName('value')[0]), '<br>'
            if get_text_node(attr.getElementsByTagName('value')[0]) != 'PersEducNat':
                continue
        # prof : doublon avec ENTPersonJointure
        #for identifier in request.getElementsByTagName('identifier'):
        #    pass
        #    print "identifiant :", get_text_node(identifier.getElementsByTagName('id')[0]), '<br>'
        dico = {}
        for attributes in request.getElementsByTagName('attributes'):
            for attr in attributes.getElementsByTagName('attr'):
                value = ''
                for val in attr.getElementsByTagName('value'):
                    if value != '':
                        # FIXME : séparateur (1)
                        value += '\n'
                    value += get_text_node(val)
                if value != '':
                    # FIXME : séparateur(2)
                    #value.replace('$', ' ')
                    # encode pour affichage uniquement
                    #print attr.getAttribute("name"), ":", value.encode('iso-8859-1'), '<br>'
                    dico[attr.getAttribute("name")] = value.encode('utf-8')
                else:
                    # attribut vide
                    pass
        try:
            if dico['PersEducNatPresenceDevantEleves'] != 'N':
                continue
            login = gen_login(dico['givenName'], dico['sn'])
            if pwdtype == '2':
                date = os.popen("/usr/bin/pwgen -1").read().strip()
            elif dico.has_key('ENTPersonDateNaissance'):
                date = gen_date(dico['ENTPersonDateNaissance'])
            else:
                date += '00000000'
            if dico.has_key('ENTPersonFonctions'):
                try:
                    fonction = dico['ENTPersonFonctions'].split('$')[2]
                    if fonction == "ENSEIGNEMENT DEVANT ELEVES":
                        # prof rescapé
                        continue
                    groupe = ok_groupe(fonction)
                except:
                    groupe = 'DomainUsers'
            if backend.is_user(login):
                messages.append("Utilisateur %s déjà existant"%login)
                continue
            if not backend.is_group(groupe):
                if not backend.add_group(groupe):
                    groupe = 'DomainUsers'
            messages.append("Ajout de l'utilisateur %s (%s)"%(login, groupe))
            if backend.add_user(login, groupe):
                backend.mod_password(login, date)
                passlines.append("%s;%s\n"%(login, date))
            else:
                messages.append("Echec d'ajout utilisateur %s"%login)
        except:
            # entrée invalide
            continue
    # écriture du fichier des mots de passe
    if passlines != []:
        passlines.insert(0,'LOGIN;PASSWORD\n')
        messages.append("** Liste des mots de passe dans %s **"%passfile)
        if os.path.isfile(passfile):
            messages.append("(ancien fichier renommé en %s.bak)"%passfile)
            os.rename(passfile, passfile+'.bak')
        try:
            f_pass = file(passfile, 'w')
            f_pass.writelines(passlines)
            f_pass.close()
        except:
            messages.append("Impossible d'écrire le fichier %s !"%filename)
            return messages
    else:
        messages.append("Aucun nouvel utilisateur n'a été créé")

    return messages

