# -*- coding: utf-8 -*-
###########################################################################
# Eole NG - 2007
# Copyright Pole de Competence Eole  (Ministere Education - Academie Dijon)
# Licence CeCill  cf /root/LicenceEole.txt
# eole@ac-dijon.fr
#
# backend.py
#
# librairie pour la gestion des utilisateurs horus
#
###########################################################################

from creole import parsedico
import backend_conf
from os import system, listdir
from os.path import join, isdir
import ldap


dico = parsedico.parse_dico()
smb_serveur = dico["smb_netbios_name"]
branche = "ou=%s,ou=%s,ou=education,o=gouv,c=fr" % (dico["numero_etab"], dico["nom_academie"])
user_filters = "(objectclass=sambaSamAccount)(objectclass=inetOrgPerson)(!(description=Computer))"
group_filters = "(objectclass=sambaGroupMapping)(objectclass=posixGroup)"
station_filters = "(description=Computer)(objectclass=posixAccount)"
share_filters = "(objectClass=sambaFileShare)"

SKEL_REP = "/usr/share/horus/skel"
HOMEBASE = "/home"


#####################
## Primitives LDAP ##
#####################

def ldapsearch(filtre, attrlist=None):
    """
    recherche dans l'annuaire
    """
    query = ldap.open(backend_conf.serveur)
    reload(backend_conf)
    query.simple_bind_s("cn=admin,o=gouv,c=fr", backend_conf.ldap_passwd)
    result = query.search_s("o=gouv,c=fr", ldap.SCOPE_SUBTREE, filtre, attrlist)
    query.unbind()
    return result


def ldapadd(dn, data):
    """
    ajout d'une entrée dans l'annuaire
    """
    query = ldap.open(backend_conf.serveur)
    reload(backend_conf)
    query.simple_bind_s("cn=admin,o=gouv,c=fr", backend_conf.ldap_passwd)
    result = query.add_s(dn, data)
    query.unbind()
    return result


def ldapdelete(dn):
    """
    suppression d'une entrée de l'annuaire
    """
    query = ldap.open(backend_conf.serveur)
    reload(backend_conf)
    query.simple_bind_s("cn=admin,o=gouv,c=fr", backend_conf.ldap_passwd)
    result = query.delete(dn)
    query.unbind()
    return result


def ldapmodify(dn, attribute, value):
    """
    modification d'une entrée de l'annuaire
    """
    if value == '':
        value = None
    query = ldap.open(backend_conf.serveur)
    reload(backend_conf)
    query.simple_bind_s("cn=admin,o=gouv,c=fr", backend_conf.ldap_passwd)
    try:
        query.modify_s(dn, [(ldap.MOD_REPLACE, attribute, value)])
    except:
        try:
            query.modify_s(dn, [(ldap.MOD_ADD, attribute, value)])
        except:
            return False
    return True
    

def authenticate(user, password):
    """
    tentative de validation d'un couple login/mdp
    """
    if user == '' or password == '':
        return False
    query = ldap.open(backend_conf.serveur)
    try:
        query.simple_bind_s("uid=%s,ou=local,ou=personnels,ou=utilisateurs,%s"%(user, branche), password)
        query.unbind()
        return True
    except ldap.INVALID_CREDENTIALS:
        return False


############################
## fonctions de recherche ##
############################

def is_user(user):
    """
    vérification de l'existance d'un utilisateur
    """
    result = ldapsearch("(&%s(uid=%s))" % (user_filters, user), ['uid'])
    if result == []:
        return False
    else:
        return True


def is_system_user(user):
    """
    indique si le login proposé est déjà un utilisateur système
    """
    user = user.lower()
    passfile = open('/etc/passwd','r')
    passbuffer = passfile.readlines()
    passfile.close()
    for ligne in passbuffer:
        if user == ligne.split(':')[0]:
            return True
    return False


def get_gid(user):
    """
    gid de l'utilisateur
    """
    result = ldapsearch("(&%s(uid=%s))" % (user_filters, user), ['gidNumber'])
    try:
        result = ldapsearch("(&%s(gidNumber=%s))" % (group_filters, result[0][1]['gidNumber'][0]), ['cn'])
        return result[0][1]['cn'][0]
    except:
        return ''


def get_attr(user, attr):
    """
    renvoi le contenu d'un attribut utilisateur
    """
    result = ldapsearch("(&%s(uid=%s))" % (user_filters, user), [attr])
    try:
        return result[0][1][attr]
    except:
        return []


def get_users():
    """
    liste des utilisateurs
    """
    result = ldapsearch("(&%s)" % user_filters, ['uid'])
    users = []
    for user in result:
        users.append(user[1]['uid'][0])
    users.sort()
    return users


def get_user_groups(user):
    """
    liste des groupes d'un utilisateur
    """
    result = ldapsearch("(&%s(memberUid=%s))" % (group_filters, user), ['cn'])
    groups = []
    for group in result:
        groups.append(group[1]['cn'][0])
    groups.sort()
    return groups


def get_user_no_groups(user):
    """
    liste des groupes dans lesquels l'utilisateur est absent
    """
    result = ldapsearch("(&%s(!(memberUid=%s))(!(cn=DomainComputers)))" % (group_filters, user), ['cn'])
    groups = []
    for group in result:
        groups.append(group[1]['cn'][0])
    groups.sort()
    return groups


def get_user_data(user):
    """
    renvoit tout les attributs de l'utilisateur en vrac
    """
    return ldapsearch("(&%s(uid=%s))" % (user_filters, user))[0][1]


def get_active_users(active=True):
    """
    liste des utilisateurs actifs (True)
    ou inactifs (False)
    """
    result = ldapsearch("(&%s)" % (user_filters), ['uid', 'sambaAcctFlags'])
    users = []
    for user in result:
        if user[1]['sambaAcctFlags'][0].count('DU'):
            if not active:
                users.append(user[1]['uid'][0])
        elif active:
            users.append(user[1]['uid'][0])
    return users


def get_workstations():
    """
    liste des postes de travail
    """
    result = ldapsearch("(&%s)" % station_filters, ['uid'])
    stats = []
    for stat in result:
        # les postes se terminent par $
        stats.append(stat[1]['uid'][0].replace("$",""))
    stats.sort()
    return stats


def is_group(group):
    """ groupe existant ? """
    result = ldapsearch("(&%s(cn=%s))" % (group_filters, group), ['cn'])
    return result != []


def get_groups():
    """
    liste des groupes
    """
    result = ldapsearch("(&%s(!(cn=DomainComputers)))" % group_filters, ['cn'])
    groups = []
    for group in result:
        groups.append(group[1]['cn'][0])
    groups.sort()
    return groups


def get_members(group):
    """
    liste des membres d'un groupe
    """
    result = ldapsearch("(&%s(cn=%s))" % (group_filters, group), ['memberUid'])
    try:
        res = result[0][1]['memberUid']
        res.sort()
        return res
    except:
        return []


def get_group_shares(group):
    """
    liste des partages liés à un groupe
    """
    result = ldapsearch("(&%s(sambaShareGroup=%s))" % (share_filters, group), ['sambaShareName'])
    list_shares = []
    for share in result:
        list_shares.append(share[1]['sambaShareName'][0])
    return list_shares


def get_special_shares():
    """ récupération des partages spéciaux, i.e. : avec une lettre réservée """
    result = ldapsearch("(&%s(sambaShareDrive=*))" % share_filters, ['sambaShareName', 'sambaShareDrive'])
    special_shares = {}
    for share in result:
        if share[1]['sambaShareDrive'][0] != '':
            special_shares[share[1]['sambaShareName'][0]] = share[1]['sambaShareDrive'][0]
    return special_shares

def get_shares():
    """ 
    récupération des partages et des lettres de lecteur
    """
    result = ldapsearch("%s" % share_filters, ['sambaShareName', 'sambaShareDrive'])
    shares = {}
    for share in result:
        if share[1].has_key('sambaShareDrive') :
            shares[share[1]['sambaShareName'][0]] = share[1]['sambaShareDrive'][0]
        else:
            shares[share[1]['sambaShareName'][0]] = 'None'
    return shares

def get_share_template(sharename=None):
    """ renvoie le modele du partage sharename """
    result = ldapsearch("(&(sambaShareName=%s)%s)" % (sharename, share_filters), ['sambaShareName', 'sambaShareModel'])
    try:
        return result[0][1]['sambaShareModel'][0]
    except:
        return ""

def get_share_templates():
    """
    retourne la liste des modèles de partage disponibles
    """
    templates = []
    files = listdir('/usr/share/horus/models/')
    for fil in files:
        if fil.endswith('.tmpl'):
            templates.append(fil[0:-5])
    if 'standard' not in templates:
        # il en faut au moins un
        templates.append('standard') 
    return templates


#######################
## fonctions d'ajout ##
#######################


def inscription(user, group):
    """
    inscription d'un utilisateur dans un groupe
    """
    res = system("/usr/sbin/smbldap-groupmod -m %s %s" % (user, group))
    return res == 0 


def add_user(login, group, groups=[], drive='U:', profile=False):
    """
    ajout d'un utilisateur
    @param login  : login de l'utilisateur
    @param group  : groupe principal de l'utilisateur
    @param groups : autres groupes de l'utilisateur
    @param drive  : lettre de lecteur pour le montage du répertoire personnel
    """
    home = join(HOMEBASE, login)
    smbhome = "\\\\%s\\%s\\perso" % (smb_serveur, login)
    script = "%s.bat" % login
    acctflags = "[U]"
    
    if profile:
        profiles = "\\\\"+smb_serveur+"\\"+login+"\\profiles"
    else:
        profiles = "''"
    
    if 'DomainUsers' not in groups :
        # bug ? and group != 'DomainUsers':
        # samba3 : tout le monde dans DomainUsers
        groups.append('DomainUsers')
    
    # gestion des groupes supplémentaires
    grouplist = ''
    for gro in groups:
        if grouplist == '':
            grouplist = '-G %s' % gro
        else:
            grouplist += ',%s' % gro
    
    res = system("/usr/sbin/smbldap-useradd -a -A 1 -m -k '"+SKEL_REP+"' -d '"+home+"' -D '"+drive+"' -F '"+profiles+"' -C '"+smbhome+"' -E '"+script+"' -g '"+group+"' "+grouplist+" -H '"+acctflags+"' -s '/bin/false' -c '"+login+"' "+login)
    return res == 0


def add_workstation(workstation):
    """
    ajoute une station de travail
    """
    res = system("/usr/sbin/smbldap-useradd -w %s" % workstation)
    return res == 0


def add_group(group):
    """
    ajout d'un groupe
    """
    res = system("/usr/sbin/smbldap-groupadd -a %s" % group)
    return res == 0


def add_share(sharename, groupe, filepath=None, drive=None, sticky=False, model="standard", sync=True):
    """
    ajout d'un partage samba associé à un groupe donné
    """
    if filepath is None:
        filepath = "/home/workgroups/"+sharename
    # construction des données ldap
    dn = "cn=smb://%s/%s,ou=local,ou=partages,%s" % (smb_serveur, sharename, branche)
    data = [("objectClass","sambaFileShare"),
    ( "cn","smb://"+smb_serveur+"/"+sharename),
    ("sambaShareName", sharename),
    ("description", sharename),
    ("sambaShareGroup", groupe),
    ("sambaFilePath", filepath),
    ("sambaShareURI", "\\\\%s\\%s" % (smb_serveur, sharename)),
    ("sambaShareModel", model),
    # ("sambaShareAdmin", "Administrateurs"),
    # ("location", "\\"),
    # ("server", smb_serveur)
    ]
    if sticky:
        data.append(("sambaShareOptions", "sticky"))
    if drive is not None:
        if not drive.endswith(":"):
            drive += ':'
        if len(drive) != 2:
            return False
        data.append(("sambaShareDrive", drive))

    try:
        ldapadd(dn, data)
    except:
        return False

    if sync:
        synchronize()
    return True


###############################
## fonctions de modification ##
###############################

def mod_user(login, homedrive=None, enable=None, profile=None, gid=None):
    """
    modification d'un utilisateur existant.
    """
    # les élément à ne pas modifier sont à None
    args = ''
    if homedrive is not None:
        args += '-D %s ' % homedrive
    
    if profile is not None:
        if profile == 0:
            args += '-F "" '
        else:
            profile = "\\\\\\\\"+smb_serveur+"\\\\"+login+"\\\\profiles"
            args += '-F %s ' % profile

    if enable is not None:
        if enable:
            args += "-J "
        else:
            args += "-I "
    if gid is not None:
        args += '-g %s' % gid
    
    if args != '':
        result = system('/usr/sbin/smbldap-usermod %s %s' % (args, login))
        if result != 0:
            return False
        if gid is not None:
            # le changement de groupe principal ne met pas à jour l'appartenance au groupe
            inscription(login, gid)
    return True


def mod_password(login, passwd):
    """ changement d'un mot de passe """
    result = system('/usr/sbin/horus-passwd %s %s >/dev/null' % (login, passwd))
    return result == 0


def set_active_users(users, active=True):
    """
    active/désactive les utilisateurs passés en paramètre
    """
    if type(users) != list:
        users = [users]
    if active:
        value = '[U]'
    else:
        value = '[DU]'
    for user in users:
        try:
            set_attr(user, 'sambaAcctFlags', value)
        except:
            return False
    return True


def set_attr(user, attr, content):
    """
    modifie un attribut utilisateur
    """
    try:
        result = ldapsearch("(&%s(uid=%s))" % (user_filters, user), [''])
        ldapmodify(result[0][0], attr, content)
        return True
    except:
        return False


def mod_share(share, drive=None, model=None):
    """
    modification d'un partage (lettre ou modèle samba)
    """
    result = ldapsearch("(&%s(sambaShareName=%s))" % (share_filters, share))
    if drive != None:
        drive = drive.upper()
        ldapmodify(result[0][0], 'sambaShareDrive', drive)
    if model != None:
        ldapmodify(result[0][0], 'sambaShareModel', model)


##############################
## fonctions de suppression ##
##############################

def desinscription(user, group):
    """
    désinscription d'un utilisateur d'un groupe
    """
    res = system("/usr/sbin/smbldap-groupmod -x %s %s" % (user, group))
    return res == 0


def del_user(user, remove_data=False):
    """
    suppression d'un utilisateur avec ou sans son répertoire personnel
    """
    if remove_data:
        res = system("/usr/sbin/smbldap-userdel -r "+user)
    else:
        res = system("/usr/sbin/smbldap-userdel "+user)
    return res == 0


def del_workstation(workstation):
    """
    suppression d'une station de travail
    """
    if not workstation.endswith('$'):
        workstation = workstation+'$'
    res = system("/usr/sbin/smbldap-userdel %s" % workstation)
    return res == 0


def del_group(group):
    """
    suppression d'un groupe
    Attention il faut au préalable avoir fait les vérifications d'usage :
       - il reste des membres
       - il reste des partages liés
    """
    res = system("/usr/sbin/smbldap-groupdel %s" % group)
    return res == 0


def del_share(share, sync=True):
    """
    suppression d'un partage
    """
    result = ldapsearch("(&%s(sambaShareName=%s))" % (share_filters, share))
    try:
        ldapdelete(result[0][0])
    except:
        return False
    if sync:
        synchronize()
    return True


##########################
## gestion des fichiers ##
##########################
def use_esu():
    """
    utilisation d'Esu
    @return True/False
    """
    for i in listdir('/home/esu/Base'):
        if i.lower() == 'esuclnt.exe':
            return True
            break
    return False

def set_acl(rep, user, group):
    """ mise en place des ACL sur un répertoire """
    cmd = '/usr/bin/setfacl -m u:%s:rwx %s' % (user, rep)
    cmd2 = '/usr/bin/setfacl -m g:%s:rwx %s' % (group, rep)
    cmd3 = '/usr/bin/getfacl --access --absolute-names %s | /usr/bin/setfacl -d -M- %s' % (rep, rep)
    system('%s;%s;%s' % (cmd, cmd2, cmd3))

def synchronize(restart=True):
    """
    ajout des partages dans le fichier smb.conf
    """    
    fic = open('/etc/sysconfig/eole/global_smb.tmpl')
    smb = [fic.read()]
    fic.close()

    templates = {}
    for tmpl in get_share_templates():
        try:
            fic = open('/usr/share/horus/models/%s.tmpl' % tmpl)
            templates[tmpl] = fic.read()
            fic.close()
        except:
            print "Erreur de lecture du template %s" % tmpl

    default = """[%(name)s]\n    inherit permissions = no\n    inherit acls = no
    readonly = no\n    preserve case = no\n    create mask = 644\n    directory mask = 755
    Write list = @%(group)s\n    Valid Users = @%(group)s @DomainAdmins\n    path = %(path)s
    comment = %(desc)s\n    Volume = %(desc)s\n\n"""

    result = ldapsearch(share_filters)
    for share in result:
        dico_share = {}
        dico_share['name'] = share[1]['sambaShareName'][0]
        dico_share['path'] = share[1]['sambaFilePath'][0]
        dico_share['group'] = share[1]['sambaShareGroup'][0]
        dico_share['desc'] = share[1]['description'][0]
        model = share[1]['sambaShareModel'][0]
        # création du répertoire si manquant
        if not dico_share['path'].startswith('%') and not isdir(dico_share['path']):
            system("mkdir -p %(path)s;chgrp %(group)s %(path)s;chmod 770 %(path)s" % dico_share)
            if share[1].has_key('sambaShareOptions'):
                if 'sticky' in share[1]['sambaShareOptions']:
                    system ("chmod 1770 %(path)s" % dico_share)
            set_acl(dico_share['path'], 'root', dico_share['group'])
        ## génération du template pour samba
        try:
            smb.append(templates[model] % dico_share)
        except KeyError:
            print "Le modèle %s n'existe pas" % model
            smb.append(default % dico_share)
        except:
            print "Erreur lors de la templatisation du modèle %s" % model
            smb.append(default % dico_share)

    # écriture du fichier final
    fic = open('/etc/samba/smb.conf', 'w')
    fic.writelines(smb)
    fic.close()
    
    # FIXME : traitements spécifiques à Horus + imprimantes

    if restart:
        system('/etc/init.d/samba restart')
